
COURSES_BASE_URL = "https://nikel.ml/api/courses?"

PROGRAMS_BASE_URL = "https://nikel.ml/api/programs?"

TEXTBOOKS_BASE_URL = "https://nikel.ml/api/textbooks?"

EXAMS_BASE_URL = "https://nikel.ml/api/exams?"

EVALS_BASE_URL = "https://nikel.ml/api/evals?"

FOOD_BASE_URL = "https://nikel.ml/api/food?"

SERVICES_BASE_URL = "https://nikel.ml/api/services?"

BUILDINGS_BASE_URL = "https://nikel.ml/api/buildings?"

PARKING_BASE_URL = "https://nikel.ml/api/parking?"


import typing
import aiohttp
import requests
from nikel_py.utils._urlparser import url_parse
from nikel_py.utils.errors import NoDataFound, ResponseError


def _get(base_url : str, query : typing.Dict = None, limit : int = 10):

    url = base_url if not query else url_parse(base_url, query, limit)


    try:

        r = requests.get(url)

        if r.status_code != 200:
            raise ResponseError

        r = r.json()

    except Exception:
        raise ResponseError


    if len(r['response']) >= 1:
        return r['response']


    else:
        raise NoDataFound






async def _async_get(base_url : str, query : typing.Dict = None, limit : int = 10):

    url = base_url if not query else url_parse(base_url, query, limit)


    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as resp:

                if resp.status != 200:
                    raise ResponseError

                r = await resp.json()

    except Exception:
        raise ResponseError


    if len(r['response']) >= 1:
        return r['response']


    else:
        raise NoDataFound


class NoDataFound(Exception):

    def __init__(self, message = "No Data was found for your query"):
        super().__init__(message)


class InvalidKey(Exception):

    def __init__(self, key : str, message = "Invalid Key Passed in: '{}'"):
        self.message = message.format(key)
        super().__init__(self.message)



class ResponseError(Exception):
    def __init__(self, message = "HTTP Request Unsuccessful. Possibly Rate limited?"):
        super().__init__(message)


import typing
from nikel_py.utils._constants import BUILDINGS_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Building:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.code = response['code']
        self.name = response['name']
        self.tags = response['tags']
        self.short_name = response['short_name']
        self.address = response['address']
        self.coordinates = response['coordinates']
        self.last_updated = response["last_updated"]


    @property
    def all_data(self):
        return self._response



class Buildings:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(BUILDINGS_BASE_URL, query, limit)

        return [Building(i) for i in r]


    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(BUILDINGS_BASE_URL, query, limit)

        return [Building(i) for i in r]


import typing
from nikel_py.utils._constants import TEXTBOOKS_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Textbook:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.isbn = response['isbn']
        self.title = response['title']
        self.edition = response['edition']
        self.author = response['author']
        self.image = response['image']
        self.price = response['price']
        self.url = response['url']
        self.courses = response['courses']
        self.last_updated = response["last_updated"]



    @property
    def all_data(self):
        return self._response



class Textbooks:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(TEXTBOOKS_BASE_URL, query, limit)

        return [Textbook(i) for i in r]




    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(TEXTBOOKS_BASE_URL, query, limit)

        return [Textbook(i) for i in r]


import typing
from nikel_py.utils._constants import FOOD_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Food:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.name = response['name']
        self.description = response['description']
        self.tags = response['tags']
        self.campus = response['campus']
        self.address = response['address']
        self.coordinates = response['coordinates']
        self.hours = response['hours']
        self.image = response['image']
        self.twitter = response["twitter"]
        self.facebook = response["facebook"]
        self.attributes = response["attributes"]
        self.last_updated = response["last_updated"]



    @property
    def all_data(self):
        return self._response



class Foods:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(FOOD_BASE_URL, query, limit)

        return [Food(i) for i in r]



    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(FOOD_BASE_URL, query, limit)

        return [Food(i) for i in r]


import typing
from nikel_py.utils._constants import EVALS_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Eval:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.campus = response['campus']
        self.name = response['name']
        self.terms = response['terms']
        self.last_updated = response["last_updated"]


    @property
    def all_data(self):
        return self._response



class Evals:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(EVALS_BASE_URL, query, limit)

        return [Eval(i) for i in r]




    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(EVALS_BASE_URL, query, limit)

        return [Eval(i) for i in r]


import typing
from nikel_py.utils._constants import COURSES_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Course:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.code = response['code']
        self.name = response['name']
        self.description = response['description']
        self.division = response['division']
        self.department = response['department']
        self.prerequisites = response['prerequisites']
        self.corequisites = response['corequisites']
        self.exclusions = response['exclusions']
        self.recommended_preparation = response['recommended_preparation']
        self.level = response['level']
        self.campus = response['campus']
        self.term = response['term']
        self.arts_and_science_breadth = response['arts_and_science_breadth']
        self.arts_and_science_distribution = response['arts_and_science_distribution']
        self.utm_distribution = response['utm_distribution']
        self.utsc_breadth = response['utsc_breadth']
        self.apsc_electives = response['apsc_electives']
        self.meeting_sections = response['meeting_sections']
        self.last_updated = response["last_updated"]


    @property
    def all_data(self):
        return self._response



class Courses:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(COURSES_BASE_URL, query, limit)

        return [Course(i) for i in r]




    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(COURSES_BASE_URL, query, limit)

        return [Course(i) for i in r]







import typing
from nikel_py.utils._constants import EXAMS_BASE_URL
from nikel_py.utils._getter import _get, _async_get


class Exam:

    def __init__(self, response):

        self._response = response

        self.id = response['id']
        self.course_id = response['course_id']
        self.course_code = response['course_code']
        self.campus = response['campus']
        self.date = response['date']
        self.start = response['start']
        self.end = response['end']
        self.duration = response['duration']
        self.sections = response['sections']
        self.last_updated = response["last_updated"]


    @property
    def all_data(self):
        return self._response



class Exams:


    @staticmethod
    def get(query : typing.Dict = None, limit : int = 10):

        r = _get(EXAMS_BASE_URL, query, limit)

        return [Exam(i) for i in r]



    @staticmethod
    async def async_get(query : typing.Dict = None, limit : int = 10):

        r = await _async_get(EXAMS_BASE_URL, query, limit)

        return [Exam(i) for i in r]


from nikel_py.utils._constants import *
from nikel_py.utils.errors import InvalidKey


# #Currently not being used but better support may be added in the future
# OPERATORS = ("!", "<", "<=", ">", ">=", "(", ")", "~")


def url_parse(base_url : str, query : dict, limit : int) -> str:

    lookup = {
        COURSES_BASE_URL : ('id', 'code', 'name', 'description', 'division', 'department', 'prerequisites', 'corequisites', 'exclusions', 'recommended_preparation', 'level', 'campus', 'term', 'arts_and_science_breadth', 'arts_and_science_distribution', 'utm_distribution', 'utsc_breadth', 'apsc_electives', 'meeting_sections', 'last_updated'),
        PROGRAMS_BASE_URL : ('id', 'name', 'type', 'campus', 'description', 'enrollment', 'completion', 'last_updated'),
        TEXTBOOKS_BASE_URL : ('id', 'isbn', 'title', 'edition', 'author', 'image', 'price', 'url', 'courses', 'last_updated'),
        EXAMS_BASE_URL : ('id', 'course_id', 'course_code', 'campus', 'date', 'start', 'end', 'duration', 'sections', 'last_updated'),
        EVALS_BASE_URL : ('id', 'name', 'campus', 'terms', 'last_updated'),
        FOOD_BASE_URL : ('id', 'name', 'description', 'tags', 'campus', 'address', 'coordinates', 'hours', 'image', 'url', 'twitter', 'facebook', 'attributes', 'last_updated'),
        SERVICES_BASE_URL : ('id', 'name', 'alias', 'building_id', 'description', 'campus', 'address', 'image', 'coordinates', 'tags', 'attributes', 'last_updated'),
        BUILDINGS_BASE_URL : ('id', 'code', 'tags', 'name', 'short_name', 'address', 'coordinates', 'last_updated'),
        PARKING_BASE_URL : ('id', 'name', 'alias', 'building_id', 'description', 'campus', 'address', 'coordinates', 'last_updated')
    }

    for i in query:

        if i not in lookup[base_url[:base_url.find("?") + 1]]:
            raise InvalidKey(i)

        if isinstance(query[i], str):

            if " " in query[i]:

                if query[i].startswith("~"):
                    query[i] = query[i].replace(" ", f"&{i}=~")

                else:
                    query[i] = query[i].replace(" ", f"&{i}=")



        base_url += f"&{i}={query[i]}"

    base_url += f"&limit={limit}" if (limit >=1 and limit <=100) else "&limit=100"

    return base_url


from setuptools import setup
from codecs import open
from os import path


with open('requirements.txt') as f:
    requirements = f.read().splitlines()




# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name="nikel-py",
    version="0.1.1",
    description="Python API Wrapper for the Nikel API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Multivalence/nikel-py",
    author="Multivalence",
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'


    ],
    packages=["nikel_py", "nikel_py.utils"],
    include_package_data=True,
    install_requires=requirements,
    python_requires='>=3.6.0'
)